/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QListView *m_listView;
   QSortFilterProxyModel *m_proxyModel;

   void addEntry();
   void removeEntry();
};

MainWindow::MainWindow()
{
   setMinimumSize(500, 600);

   QLabel *label = new QLabel();
   label->setText("List View");
   label->setAlignment(Qt::AlignHCenter);

   QFont font = label->font();
   font.setPointSize(11);
   label->setFont(font);

   QStringList data = { "apple", "watermelon", "strawberry", "nectarine", "guava", "orange" };

   QStringListModel *model = new QStringListModel(this);
   model->setStringList(data);

   m_proxyModel = new QSortFilterProxyModel(this);
   m_proxyModel->setSourceModel(model);
   m_proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
   m_proxyModel->setDynamicSortFilter(true);
   m_proxyModel->sort(0, Qt::AscendingOrder);

   m_listView = new QListView();
   m_listView->setModel(m_proxyModel);
   m_listView->setEditTriggers(QAbstractItemView::DoubleClicked);

   QModelIndex index = m_proxyModel->index(0, 0);
   m_listView->setCurrentIndex(index);

   QPushButton *pb_plus = new QPushButton();
   pb_plus->setIcon(QIcon("../plus.png"));

   QPushButton *pb_minus = new QPushButton();
   pb_minus->setIcon(QIcon("../minus.png"));

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   QHBoxLayout *layout = new QHBoxLayout();
   layout->addStretch();
   layout->addWidget(pb_plus);
   layout->addSpacing(8);
   layout->addWidget(pb_minus);
   layout->addSpacing(30);
   layout->addWidget(pb_close);
   layout->addStretch();

   QVBoxLayout *layout_main = new QVBoxLayout(this);
   layout_main->setContentsMargins(20, 20, 20, 15);

   layout_main->addWidget(label);
   layout_main->addSpacing(10);
   layout_main->addWidget(m_listView);
   layout_main->addSpacing(10);
   layout_main->addLayout(layout);

   connect(pb_plus,  &QAbstractButton::clicked, this, &MainWindow::addEntry);
   connect(pb_minus, &QAbstractButton::clicked, this, &MainWindow::removeEntry);

   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);
}

void MainWindow::addEntry()
{
   // turn sorting off
   m_proxyModel->setDynamicSortFilter(false);
   m_proxyModel->sort(-1);

   int row = m_listView->currentIndex().row();
   m_proxyModel->insertRows(row, 1);

   // edit new element
   QModelIndex index = m_proxyModel->index(row, 0);
   m_listView->setCurrentIndex(index);
   m_listView->edit(index);

   // turn sorting on
   m_proxyModel->setDynamicSortFilter(true);
   m_proxyModel->sort(0, Qt::AscendingOrder);
}

void MainWindow::removeEntry()
{
   m_proxyModel->removeRows(m_listView->currentIndex().row(), 1);
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
